//------------------------------------------------------------------------------
//
//  npdata.cpp
//  npe
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------


#include "npdata.h"
#include "npctrl.h"
#include "data/npplot.h"
#include "io/npkey.h"

#include <stdlib.h>

void* gData;

void npInitDataMap (void* dataRef);
void npInitDataIO (void* dataRef);
void npInitDataCtrl (void* dataRef);


//------------------------------------------------------------------------------
void npInitDataMap(void* dataRef)
{
	pData data = (pData) dataRef;

	NPplotPtr plot			= NULL;
	NPcameraPtr cameraData	= NULL;
	NPpointsPtr pointsData	= NULL;
	NPpinPtr pinData		= NULL;
	NPvideoPtr videoData	= NULL;
	NPgridPtr gridData		= NULL;

	data->map.plotCount		= 0;
	data->map.plotIndex		= 0;
	npInitPlot (dataRef);

	data->map.plotChildPtr	= NULL;
	data->map.branchLevel	= 0;

	npPlotNew (kPlotDefault, dataRef);			//global parent at index = 0

	// setup our primary camera
	plot = npPlotNew (kPlotCamera, dataRef);	//camera at index = 1
	cameraData = (NPcameraPtr) plot->data;
	
	plot->translate.x = 0.0f;
	plot->translate.y = -4.0f;
	plot->translate.z = 5.0f;
	
	plot->rotate.angle = 0.0f;
	plot->rotate.x = 0.0f;
	plot->rotate.y = 1.0f;
	plot->rotate.z = 0.0f;

	plot->triggerOnLow.z = true;				//prevents going below surface
	plot->triggerLevelLow.z = 1.5f;

	// setup objects
	npPlotNew (kPlotGrid, dataRef);

	npDataPreset (1, dataRef);

	data->map.plotIndex = 1;					//select the primary camera
}


//------------------------------------------------------------------------------
void npInitDataIO(void* dataRef)
{
	pData data = (pData) dataRef;

	npInitKey (dataRef);

	data->io.channelCount = 1;			// the first channel is the sample time

	data->io.drawMenu = true;
	data->io.clearFrameBuffer = true;

	data->io.axes.x = true;
	data->io.axes.y = true;
	data->io.axes.z = true;

	data->io.size = 0;
}


//------------------------------------------------------------------------------
void npInitDataCtrl(void* dataRef)
{
	
}


//------------------------------------------------------------------------------
void npInitData (int argc, char* argv[], void* dataRef)
{
	pData data = (pData) malloc (sizeof(Data));
	
	npInitDataMap (data);
	npInitDataIO (data);
	npInitDataCtrl (data);

	dataRef = gData = data;

	// create a fixed array of global data pointers that is thread safe,		ZZ
}


//------------------------------------------------------------------------------
void npCloseData()
{
	pData data = (pData) gData;
	
	free (data);
}


//------------------------------------------------------------------------------
void* npDataGetDataRef()
{
	return gData;
}


//------------------------------------------------------------------------------
void npDataUpdate (void* dataRef)
{

}

// loads preset scene using currently selected object as the origin
// if alt is pressed (ie ALT+5) it loads preset at the global origin
//------------------------------------------------------------------------------
void npDataPreset (int preset, void* dataRef)
{
	int i = 0;
	int j = 0;

	GLfloatXYZ	translateOffset;
	GLfloatXYZ	rotateOffset;
	GLfloatXYZ	scaleOffset;

	pData data = (pData) dataRef;
	NPplotPtr plot = data->map.plot[data->map.plotIndex];

	NPplotPtr plotChild = NULL;

	NPpinPtr pinData = NULL;

	// don't offset if alt key held or camera selected
	if (data->io.key.modAlt || data->map.plotIndex == 1)
	{
		translateOffset.x = 0.0f;
		translateOffset.y = 0.0f;
		translateOffset.z = 0.0f;

		rotateOffset.x = 0.0f;
		rotateOffset.y = 0.0f;
		rotateOffset.z = 0.0f;

		scaleOffset.x = 0.0f;
		scaleOffset.y = 0.0f;
		scaleOffset.z = 0.0f;
	}
	else
	{
					// add code to calc translate, rotate position based on
					// global coordinates think of GL order of operation,	debug zz
		translateOffset.x = plot->translate.x;
		translateOffset.y = plot->translate.y;
		translateOffset.z = plot->translate.z;

		rotateOffset.x = plot->rotateRad.x;
		rotateOffset.y = plot->rotateRad.y;
		rotateOffset.z = plot->rotateRad.z;

		scaleOffset.x = plot->scale.x;
		scaleOffset.y = plot->scale.y;
		scaleOffset.z = plot->scale.z;
	}

	switch (preset)
	{
		case kGraphPresetOne :
			plot = npPlotNew (kPlotSurface, dataRef);
			
			plot->translate.x += translateOffset.x;
			plot->translate.y += translateOffset.y;
			plot->translate.z += translateOffset.z;

/*			plot->rotateRad.x = rotateOffset.x;
			plot->rotateRad.y = rotateOffset.y;
			plot->rotateRad.z = rotateOffset.z;

			plot->scale.x = scaleOffset.x;
			plot->scale.y = scaleOffset.y;
			plot->scale.z = scaleOffset.z;
*/
			npPlotNew (kPlotPin, dataRef);
			break;

		case kGraphPresetTwo :

			// draw 6x6 grid of pins
			for(i = -3; i < 3; i++)
				for(j=-3; j < 3; j++)
				{
					// create the pin trunk
					plot = npPlotNew (kPlotPin, dataRef);
					plot->translate.x = i * 10.0f;
					plot->translate.y = j * 10.0f;

					plot->translate.x += translateOffset.x;
					plot->translate.y += translateOffset.y;
					plot->translate.z += translateOffset.z;

					// create child plot toroids...
					plot = npPlotNewChild (plot, dataRef);
					npPlotNewChild (plot, dataRef);
					npPlotNewChild (plot, dataRef);
					plot = npPlotNewChild (plot, dataRef);
					npPlotNewChild (plot, dataRef);
					plot = npPlotNewChild (plot, dataRef);
					npPlotNewChild (plot, dataRef);
				}			
			break;
		case kGraphPresetThree :
			npPlotNew (kPlotSurface, dataRef);
			npPlotNew (kPlotPin, dataRef);
			break;
		case kGraphPresetFour : 
			break;

		case kGraphPresetFive : break;
		case kGraphPresetSix : break;
		case kGraphPresetSeven : break;
		case kGraphPresetEight : break;

		default : break;
	}

	data->map.plotIndex = 1;			// select the camera
}